# Script to study performance of the non-parametric bootstrap when computing
# confidence intervals for the mean and SD of a bimodal distribution.
# 
# Author: Will Guthrie, 2013-08-16
# Last Change: 2013-08-16
# Hacked by DWF 2013-08-30 12:52
###############################################################################

options(warn=1)
library(boot)

# This does not speed it up AFAICT.
# library(multicore)

args <- commandArgs(trailingOnly=T)
if (length(args) != 1) stop("bad command line")
f1 <- as.numeric(args[1])
print(f1)

# For quick run, set n.s to 100; for accurate answers, set at least 1000.

n.d <- 1000     # Sample size N
n.s <- 1000     # Number of iterations of the whole shebang
n.b <- 10000    # Bootstrap M
mu1 <- 30
sigma1 <- 1
mu2 <- 10
sigma2 <- 1

# Parameters are the original data and indices to sample (maybe repeated).
# First return is the statistic of interest (sd).
# Second return used only for bootstrap-t, which DNU yet.
weirdsd <- function(data, indices) {
  sd(data[indices])
}

# compute true parameter values for underlying bimodal distribution
# cf https://en.wikipedia.org/wiki/Mixture_distribution
f2 <- 1-f1
mu <- f1*mu1+f2*mu2
sigma <- sqrt(f1*((mu1-mu)^2+sigma1^2)+f2*((mu2-mu)^2+sigma2^2))

# generate n.s sets of "original" data of size n.d 
i1 <- rbinom(n.s*n.d,1,f1)
i2 <- 1-i1
y <- matrix(i1*rnorm(n.d*n.s,mu1,sigma1)+i2*rnorm(n.d*n.s,mu2,sigma2),ncol=n.d)
i1m <- matrix(i1,ncol=n.d)

# generate n.s bootstrap (MCM) confidence intervals using the n.s original sets of data
# size of each bootstrap distribution is n.b
sigma.int <- matrix(NA,ncol=8,nrow=n.s)
colnames(sigma.int) <- c("normlo","normhi","basiclo","basichi","perclo","perchi","bcalo","bcahi")

hole_count = 0
for(i in 1:n.s) {

  ab20 <- sum(i1m[i,])
  hole_count = hole_count + (ab20==0)

  # This does not speed it up AFAICT.
  # ,parallel="multicore"
  bootobj <- boot(y[i,],weirdsd,n.b)
  bootcis <- boot.ci(bootobj,type=c("norm","basic","perc","bca"))

  sigma.int[i,1] <- bootcis$normal[2]
  sigma.int[i,2] <- bootcis$normal[3]
  sigma.int[i,3] <- bootcis$basic[4]
  sigma.int[i,4] <- bootcis$basic[5]
  sigma.int[i,5] <- bootcis$percent[4]
  sigma.int[i,6] <- bootcis$percent[5]
  sigma.int[i,7] <- bootcis$bca[4]
  sigma.int[i,8] <- bootcis$bca[5]

  print(paste("Completed interval computations for",i,"of",n.s,"data sets."))
}

covs <- matrix(NA,ncol=11,nrow=1)
colnames(covs) <- c("f1","sigma","holes","normatt","normu","basicatt","basicu","percatt","percu","bcaatt","bcau")

# compute attained and report coverage for sigma
covs[1,1] <- f1
covs[1,2] <- sigma
covs[1,3] <- hole_count
for (m in 0:3) {
  sigma.cov <- sum(sigma.int[,1+2*m]<=sigma & sigma<=sigma.int[,2+2*m])/n.s*100
  covs[1,4+2*m] <- sigma.cov
  covs[1,5+2*m] <- qnorm(0.975)*sqrt(sigma.cov*(100-sigma.cov)/n.s)
}

fnam <- sprintf("sigmaint%.4f.txt", f1)
statsout <- file(fnam,open="wt")
write.table(sigma.int,file=statsout,quote=F,row.names=F)
close(statsout)

fnam <- sprintf("covs%.4f.txt", f1)
covsout <- file(fnam,open="wt")
write.table(covs,file=covsout,quote=F,row.names=F)
close(covsout)
