/* proportionate.c  2013-05-15 15:51
   Reference executable for examining self-time variability vs. the
   proportion of self time, enhanced version.

   The first one made single invocations of functions that ran for different
   amounts of time.  This one accomplishes proportionality by instead varying
   the number of invocations of functions that run the same length of time
   and allows the balance of function length versus iterations to be adjusted.

   The first command-line argument is a proportion, 0.0 <= p <= 1.0.
   fn1 will get approximately p proportion of self-time.
   fn2 will get approximately 1-p.
   Only multiples of 0.1 are supported currently.

   The second command-line argument is an integer, 1 <= d <= bignum.
   This is the number of cycles to make in the main program, with cycles
   in the leaf functions being reduced commensurately.

   There is an additional factor of 10 conversion resulting from the manner
   in which the proportionality is implemented in the main program.

   A third case not implemented here is where the functions still have
   different run times but the balance of function length versus iterations
   is adjusted.

   Windows spin 2013-06-06 15:51
*/

// Configurable TOTALLOOP:  how many iterations of the floating-point
// busy-work will ultimately be done.
#ifndef TOTALLOOP
#define TOTALLOOP 200000000
#endif

#include <stdint.h>
#define PRIu64 "I64u"
#define SCNu64 "I64u"
#define _USE_MATH_DEFINES
#include <math.h>
#include <stdio.h>

uint64_t accumulator=1;
double   adder=0;

#pragma auto_inline(off)

void fn1 (uint64_t loopcount) {
  for (uint64_t looper=0; looper<loopcount; ++looper)
    adder += M_PI*3, accumulator = accumulator*3 + adder;
}

void fn2 (uint64_t loopcount) {
  for (uint64_t looper=0; looper<loopcount; ++looper)
    adder += M_PI*3, accumulator = accumulator*3 + adder;
}

int main (int argc, char **argv) {
  double   p = -1;
  uint64_t d =  0;
  if (argc != 3 ||
      sscanf(argv[1],"%lf",&p) != 1 || p < 0.0 || p > 1.0 ||
      sscanf(argv[2],"%" SCNu64,&d) != 1 || d == 0)
    fprintf (stderr, "Usage: proportionate p d   (0.0 <= p <= 1.0)\n");
  else {
    uint64_t loopcount = TOTALLOOP / d / 10;
    uint8_t px10 = p * 10;
    uint8_t mpx10 = 10 - px10;
    for (uint64_t l1=0; l1<d; ++l1) {
      for (uint8_t l2=0; l2<px10; ++l2)
        fn1(loopcount);
      for (uint8_t l2=0; l2<mpx10; ++l2)
        fn2(loopcount);
    }
    printf ("%" PRIu64 " %f\n", accumulator, adder);
  }
  return 0;
}
