# 2013-08-21 17:07
# Rscript extrapolation.r
#
# Hacked version of Will's regression-analysis-short-runs.R script to make
# the following changes:
#  - Use new data with R table input format (incidentally eliminates a bug
#    related to the assumed fixed ordering of functions in the old input).
#  - Use absolute sample counts not relative self time.
#  - Run multiple plots.
#  - Run multiple outputs.
#  - Add error bars on the means.
#
# Based on:
#
# Script to perform an ordinary least squares (OLS) linear fit of a
# straight-line model to profiling data from the Perf profiling tool.  Model
# is fit to the data from functions 2 through 5 and used to predict the
# result for function 1.  The observed and predicted results are then
# compared using a 95 % prediction interval for the relative measured
# execution time spent in function 1.  Note that the standard deviations of
# means were considered for use in a weighted least squares (WLS) fit, but
# the residual standard deviation from the fit of the straight-line model is
# much larger, negating the need for WLS.
#
# Author: Will Guthrie, 2012-12-19
# Last Change: 2012-12-19
###############################################################################

# This is inefficient since it recalculates everything for pdf and png.
dofile <- function(infilename,outfilename) {
  # The input now has 7 columns: function1 through function5, then main and
  # "other."  You can use the means or you can use the sums; it's just a
  # factor of 100.
  ret <- read.table(infilename,header=T)
  ret <- ret[,1:5]
  mn.ret    <- colMeans(ret)
  sd.ret    <- apply(ret,2,sd)
  count.ret <- nrow(ret)

  # 95% coverage interval for the means
  alpha <- 0.05
  dof <- count.ret - 1
  semdiv <- sqrt(count.ret)
  k <- qt(1-alpha/2,dof)
  meanlo <- mn.ret - k * sd.ret / semdiv
  meanhi <- mn.ret + k * sd.ret / semdiv

  fnc     <- 1:5
  lf2345  <- lm(mn.ret~fnc,subset=(2:5))
  plf2345 <- predict(lf2345)
  plf12   <- predict(lf2345,data.frame(fnc=1:2))
  plf1    <- predict(lf2345,data.frame(fnc=1),se.fit=T)

  if (substr(outfilename,nchar(outfilename),nchar(outfilename)) == "f") {
    pdf(outfilename)
    arrlen <- 0.12
  } else {
    png(outfilename,width=800,height=800,units="px",pointsize=16)
    arrlen <- 0.15
  }

  op <- par(mar=c(4,4,0,0))
  plot(fnc,mn.ret,col=c("black"),pch=17,cex=1.5,
    xlim=c(0.5,5.5),ylim=c(min(mn.ret)*0.5,max(mn.ret,plf1$fit)*1.1),
    xlab="Function number",ylab="Mean sample count")

  # Standard uncertainty of the means
  arrows(fnc,meanlo,fnc,meanhi,angle=90,code=3,length=arrlen,col="black")

  lines(2:5,plf2345,col="orange",lwd=3)
  lines(1:2,plf12,col="red",lwd=3,lty=3)
  segments(0.95,plf1$fit,1.05,plf1$fit,col="red",lwd=3)
  U.pred <- qt(0.975,2)*sqrt(plf1$se.fit^2+plf1$residual.scale^2)
  lpb <- plf1$fit-U.pred
  upb <- plf1$fit+U.pred
  segments(1,lpb,1,upb,col="red",lwd=3)
  legend("topright",
    c("Observed data","Model fit to data","Prediction","Prediction interval"),
    pch=c(17,-1,-1,-1),cex=1,lty=c(0,1,3,1),lwd=3,
    col=c("black","orange","red","red"))
  dev.off()
}

dofile("val1_original/val1_perf_Rtable.txt","extrapolation_perf_original.pdf")
dofile("val1_forward/val1_perf_Rtable.txt","extrapolation_perf_forward.pdf")
dofile("val1_forward/val1_gprof_Rtable.txt","extrapolation_gprof_forward.pdf")
dofile("val1_original/val1_perf_Rtable.txt","extrapolation_perf_original.png")
dofile("val1_forward/val1_perf_Rtable.txt","extrapolation_perf_forward.png")
dofile("val1_forward/val1_gprof_Rtable.txt","extrapolation_gprof_forward.png")
